﻿namespace U2UConsult.RotationManipulation.Sample
{
    using System;
    using Windows.System;
    using Windows.UI.Input;
    using Windows.UI.Xaml.Controls;
    using Windows.UI.Xaml.Input;
    using Windows.UI.Xaml.Media;

    public sealed partial class MainPage : Page
    {
        public MainPage()
        {
            this.InitializeComponent();

            // Code behind version of the comma-separated list in XAML.
            // this.Right.ManipulationMode = ManipulationModes.TranslateX | ManipulationModes.TranslateY;
        }

        // Called when the image is ready to be rendered. ActualHeight and ActualWidth are known.
        private void Image_Opened(object sender, Windows.UI.Xaml.RoutedEventArgs e)
        {
            Image img = sender as Image;
            RotateTransform rt = img.RenderTransform as RotateTransform;
            rt.CenterX = img.ActualWidth / 2;
            rt.CenterY = img.ActualHeight / 2;
        }

        // Called when rotate by touch.
        private void Left_ManipulationDelta(object sender, ManipulationDeltaRoutedEventArgs e)
        {
            this.LeftRotateTransform.Angle += e.Delta.Rotation;
        }

        // Called when rotate by mousewheel.
        private void Left_PointerWheelChanged(object sender, PointerRoutedEventArgs e)
        {
            bool shift = (e.KeyModifiers & VirtualKeyModifiers.Shift) == VirtualKeyModifiers.Shift;
            bool ctrl = (e.KeyModifiers & VirtualKeyModifiers.Control) == VirtualKeyModifiers.Control;

            if (shift && ctrl)
            {
                var delta = PointerPoint.GetCurrentPoint(e.Pointer.PointerId).Properties.MouseWheelDelta;
                // With my mouse, delta is a multiple of 30.
                this.LeftRotateTransform.Angle -= delta / 15;
            }
        }

        // Called by touch and mouse.
        private void Right_ManipulationDelta(object sender, ManipulationDeltaRoutedEventArgs e)
        {
            // Uncomment to remove inertia.
            //if (e.IsInertial)
            //{
            //    return;
            //}

            // Alternatively, use Triangle Cosines Law.
            // It uses just one trigonometric function (Acos), but you first need to calculate the lengths of all sides.

            var x = this.RightRotateTransform.CenterX - e.Position.X;
            var y = this.RightRotateTransform.CenterY - e.Position.Y;
            
            double a1 = Math.Atan(y / x);
            double a2 = Math.Atan((e.Delta.Translation.Y - y) / (x - e.Delta.Translation.X));

            this.RightRotateTransform.Angle += a1 - a2;
        }
    }
}
